#!/usr/bin/wish
# -------------------------------------------------------
#                    TRIANGLE TILING
# -------------------------------------------------------
#
# ********************************************************************
# This program was originally written by Charlie Gunn, and adapted
# at the Geometry Center (http://www.geom.umn.edu) for use at the
# Science Museum of Minnesota by Tamara Munzner, Stuart Levy, and
# Olaf Holt.
# The port to Linux was carried out by Atractor - Matemtica
# Interactiva (http://www.fc.up.pt/atractor).
# This adaptation was made by Pedro Ribeiro (pagina.de/pedroribeiro)
# ********************************************************************
#
# -------------------------------------------------------
#
#                  (Graphical Interface)
#
# -------------------------------------------------------

lappend auto_path . 
package require tritile

# -------------------------------------------------------
# Variables in namespace tritile
# -------------------------------------------------------

namespace eval tritile {
    variable delay 75   ; # Refresh rate of shape
    
    variable xdim 292   ; # Canvas x dimension
    variable ydim 283   ; # Canvas x dimension
    variable size 270   ; # Triangle shape size

    variable px         ; # Present point x coord. 
    variable py         ; # Present point y coord. 
    variable lx -1      ; # Last point x coord.
    variable ly -1      ; # Last point y coord.

    variable c_shape    ; # Groups's Curved shapes
    variable special    ; # Group's special points

    variable text       ; # Text to use on interface
    variable legend     ; # Legends of predifined special points

    variable aboutmsg   ; # Text to use on about message

    variable language   ; # Current language;
}

# -------------------------------------------------------
# Initializations
# -------------------------------------------------------

# Create the window with the toolbar
proc tritile::init_main {} {
    variable px
    variable py
    variable xdim
    variable ydim

    toplevel .main
    wm geometry .main =300x768+0-0
    wm overrideredirect .main 1

    frame .main.angle -width 300 -height 335 -borderwidth 3 -relief solid \
	    -bg #999999
    frame .main.shape -width 300 -height 315 -borderwidth 3 -relief solid \
	    -bg #999999
    frame .main.see -width 300 -height 124 -borderwidth 3 -relief solid \
	    -bg #999999
    place .main.angle -x 0 -y 0
    place .main.shape -x 0 -y 332
    place .main.see -x 0 -y 644

    # ------------------------------------------------
 
    label .main.angle.t -textvar tritile::text(0) -width 32 -bg #266d95 \
	    -font "Helvetica 16 bold" -relief raised
    place .main.angle.t -x 0 -y 0
    label .main.angle.euc -textvar tritile::text(1) -bg #999999 -font "Helvetica 14 bold"
    place .main.angle.euc -x 10 -y 50
    label .main.angle.sph -textvar tritile::text(2) -bg #999999 -font "Helvetica 14 bold"
    place .main.angle.sph -x 10 -y 255

    image create photo pi -file images/pi.gif
    for {set i 3} {$i<=6} {incr i} {
	image create photo pi$i -file images/pi$i.gif
	radiobutton .main.angle.pi$i -image pi$i -indicatoron 0 -value $i \
		-variable angle -command "tritile::group $i" \
		-selectcolor #ffff91 -bg #bbbbbb
	label .main.angle.lpi$i -bg #999999 -font "Helvetica 12 bold" \
		-text "/ $i = [expr 180/$i]"
	label .main.angle.ppi$i -bg #999999 -image pi
    }

    place .main.angle.pi6 -x 105 -y 28
    place .main.angle.ppi6 -x 112 -y 106
    place .main.angle.lpi6 -x 129 -y 106
    place .main.angle.pi5 -x 5 -y 136
    place .main.angle.ppi5 -x 12 -y 214
    place .main.angle.lpi5 -x 29 -y 214
    place .main.angle.pi4 -x 105 -y 136
    place .main.angle.ppi4 -x 112 -y 214
    place .main.angle.lpi4 -x 129 -y 214
    place .main.angle.pi3 -x 205 -y 136
    place .main.angle.ppi3 -x 212 -y 214
    place .main.angle.lpi3 -x 229 -y 214

    frame .main.angle.divider -bg black -width 294 -height 1
    place .main.angle.divider -x 0 -y 128

    image create photo flat -file images/flat.gif
    image create photo round -file images/round.gif
    frame .main.angle.poli -bg #999999
    frame .main.angle.sphere -bg #999999
    radiobutton .main.angle.poli.b -image flat -indicatoron 0 \
	    -selectcolor yellow -selectcolor #ffff91 -bg #bbbbbb \
	    -variable type -value 0 -width 80 -command "tritile::curve 5"
    label .main.angle.poli.l -textvar tritile::text(3) -bg #999999
    radiobutton .main.angle.sphere.b -image round -indicatoron 0 \
	    -selectcolor yellow -selectcolor #ffff91 -bg #bbbbbb \
	    -variable type -value 1 -width 80 -command "tritile::curve 4"
    label .main.angle.sphere.l -textvar tritile::text(4) -bg #999999
    pack .main.angle.poli.b .main.angle.poli.l -side top
    place .main.angle.poli -x 105 -y 240
    pack .main.angle.sphere.b .main.angle.sphere.l -side top
    place .main.angle.sphere -x 205 -y 240    

    # ------------------------------------------------
    
    label .main.see.t -textvar tritile::text(6) -width 32 \
	    -bg #266d95 -font "Helvetica 16 bold" -relief raised
    place .main.see.t -x 0 -y 0
    
    image create photo both -file images/both.gif
    image create photo edges -file images/edges.gif
    image create photo faces -file images/faces.gif
    image create photo neither -file images/neither.gif
    
    frame .main.see.both -bg #999999
    radiobutton .main.see.both.b -image both -indicatoron 0 \
	    -selectcolor yellow -selectcolor #ffff91 -bg #bbbbbb \
	    -variable see -value 0 -command "ChangeSee 2"
    label .main.see.both.l -textvar tritile::text(7) -bg #999999
    pack .main.see.both.b .main.see.both.l -side top

    frame .main.see.neither -bg #999999
    radiobutton .main.see.neither.b -image neither -indicatoron 0 \
	    -selectcolor yellow -selectcolor #ffff91 -bg #bbbbbb \
	    -variable see -value 1 -command "ChangeSee 3"
    label .main.see.neither.l -textvar tritile::text(8) -bg #999999
    pack .main.see.neither.b .main.see.neither.l -side top

    frame .main.see.edges -bg #999999
    radiobutton .main.see.edges.b -image edges -indicatoron 0 \
	    -selectcolor yellow -selectcolor #ffff91 -bg #bbbbbb \
	    -variable see -value 2 -command "ChangeSee 1"
    label .main.see.edges.l -textvar tritile::text(9) -bg #999999
    pack .main.see.edges.b .main.see.edges.l -side top

    frame .main.see.faces -bg #999999
    radiobutton .main.see.faces.b -image faces -indicatoron 0 \
	    -selectcolor yellow -selectcolor #ffff91 -bg #bbbbbb \
	    -variable see -value 3 -command "ChangeSee 0"
    label .main.see.faces.l -textvar tritile::text(10) -bg #999999
    pack .main.see.faces.b .main.see.faces.l -side top
    
    place .main.see.both -x 5 -y 30
    place .main.see.neither -x 78 -y 30
    place .main.see.edges -x 151 -y 30
    place .main.see.faces -x 224 -y 30

    # ------------------------------------------------
    
    label .main.shape.t -textvar tritile::text(5) -width 32 \
	    -bg #266d95 -font "Helvetica 16 bold" -relief raised
    place .main.shape.t -x 0 -y 0

    canvas .main.shape.c -width $xdim -height $ydim -bg #999999
	    # -borderwidth 10 -relief sunken
    place .main.shape.c -x 0 -y 24

    .main.shape.c create polygon 0 0 1 1 1 0 -fill "" -width 2 \
	    -outline black -tag shape
    
   .main.shape.c create polygon 0 0 1 1 1 0 -fill "#dda933" -width 2 \
	    -outline black -tag orange
    .main.shape.c create polygon 0 0 1 1 1 0 -fill "#bb44ee" -width 2 \
	    -outline black -tag magenta
    .main.shape.c create polygon 0 0 1 1 1 0 -fill "#55ffdd" -width 2 \
	    -outline black -tag green


    for {set i 0} {$i<7} {incr i} {
	.main.shape.c create oval 0 0 1 1 -fill black -tag sp$i
    }

    image create photo pi_blue -file images/pi_blue.gif
    .main.shape.c create image 40 40 -image pi_blue -tag pi_blue
    .main.shape.c create text 53 33 -text "/ 4 radians = 45" -anchor nw\
	    -font "Helvetica 16 bold" -fill #0202fe -tag angle1
    .main.shape.c create text 20 255 -text "90" -anchor nw\
	    -font "Helvetica 16 bold" -fill black -tag angle2
    .main.shape.c create text 215 255 -text "60" -anchor nw\
	    -font "Helvetica 16 bold" -fill black -tag angle3
    .main.shape.c create rectangle 225 152 262 153 -fill black 
    .main.shape.c create text 225 100 -anchor nw -font "Helvetica 16 bold" \
	    -fill black -tag sum -text "45\n60\n+ 90\n195"\
	    -justify right
    
    
    .main.shape.c create rectangle 96 124 102 130 -fill "" \
	    	    -width 2 -outline #0202fe -tag point
    
    bind .main.shape.c <Button> "set tritile::px %x ; set tritile::py %y"
    bind .main.shape.c <Button1-Motion> \
	    "set tritile::px %x ; set tritile::py %y"


    # ------------------------------------------------

    reset
    shape
}

# Create top window
proc tritile::init_top {} {
    toplevel .top -bg #595958
    wm geometry .top =724x40+300-728
    wm overrideredirect .top 1

    label .top.text -bg #595959 -fg #ffff91 -textvar tritile::text(11) \
	    -font "Helvetica 16 bold"
    pack .top.text -pady 5 
}

# Create bottom, including the legend
proc tritile::init_bottom {} {
    global argv

    toplevel .bottom -bg #595959 
    wm geometry .bottom =724x105+300-0
    wm overrideredirect .bottom 1

    frame .bottom.title -bg black -width 724 -height 43    
    button .bottom.title.about -text "About..." -command "tritile::about"
    label .bottom.title.tt -textvar tritile::text(13) -bg Black -fg #ffff91  \
	    -font "Helvetica 20 bold"
    place .bottom.title.tt -x 15 -y 10

    if {![string compare $argv "-exitbutton"]} then {
	button .bottom.title.exit -text "Exit" -command "puts (quit); exit"
	place .bottom.title.exit -relx 1 -anchor ne -x -5 -y 10
	place .bottom.title.about -relx 1 -anchor ne -x -65 -y 10
    } else {
	place .bottom.title.about -relx 1 -anchor ne -x -28 -y 10
    }
    
    frame .bottom.lg -bg #595959
    image create photo flag -file images/flags/english.gif
    label .bottom.lg.lang -text "Change Language" -bg #595959 -fg #ffff91
    button .bottom.lg.flag -image flag -command "tritile::change_language"

    button .bottom.reset -textvar tritile::text(12) -command "tritile::reset"
    label .bottom.legend -text "" \
	    -bg #595959 -fg #ffff91 -font "Helvetica 16 bold"
    
    pack .bottom.title -side bottom
    pack .bottom.legend -side bottom -pady 20
    place .bottom.lg -relx 1 -anchor ne -x -10 -y -0
    place .bottom.reset -x 15 -y 18

    pack .bottom.lg.flag .bottom.lg.lang -side top
}

# Create about window
proc tritile::init_about  {} {
    variable aboutmsg

    toplevel .about -bg black
    wm geometry .about =724x768+300-0
    wm overrideredirect .about 1
    
    label .about.title -text "About..." -bg black -fg #ffff91  \
	    -font "Helvetica 22 bold"
    place .about.title -x 320 -y 100

    # ----- Read about message ----------------
    set fid [open "language/about.txt"]
    for {set i 0} {![eof $fid]} {incr i} {
	gets $fid aboutmsg($i)
    }
    close $fid

    frame .about.msg -bg Black
    for {set i 0} {$i<[array size aboutmsg]} {incr i} {
	label .about.msg.l$i -textvar tritile::aboutmsg($i) -bg Black -fg White \
		-font "Helvetica 14 bold"
	pack .about.msg.l$i
    }
    place .about.msg -x 100 -y 220 
    
    button .about.but -text "Return to program" \
	    -command "grab release .about; wm withdraw .about; \
	    wm deiconify .top; wm deiconify .bottom"
    place .about.but -x 300 -y 550

    wm withdraw .about
}

# Read data of curved shape and special points coords.
proc tritile::init_shape {} {
    variable c_shape
    variable special
    
    set fid [open "props/shape.pts"]

    # ----- Read data from pi/5 -----
    gets $fid temp ; gets $fid temp
    set c_shape(5) ""
    gets $fid npoints
    for {set i 0} {$i<$npoints} {incr i} {
	gets $fid temp
	set temp [tri2coord [lindex $temp 0] [lindex $temp 1]]
	eval "lappend c_shape(5) $temp"
    }
    gets $fid temp
    for {set i 0} {$i<7} {incr i} {
	gets $fid temp
	set special(5,$i) [tri2coord [lindex $temp 0] [lindex $temp 1]]
    }

    # ----- Read data from pi/4 -----
    gets $fid temp ; gets $fid temp
    set c_shape(4) ""
    gets $fid npoints
    for {set i 0} {$i<$npoints} {incr i} {
	gets $fid temp
	set temp [tri2coord [lindex $temp 0] [lindex $temp 1]]
	eval "lappend c_shape(4) $temp"
    }
    gets $fid temp
    for {set i 0} {$i<7} {incr i} {
	gets $fid temp
	set special(4,$i) [tri2coord [lindex $temp 0] [lindex $temp 1]]
    }

    # ----- Read data from pi/3 -----
    gets $fid temp ; gets $fid temp
    set c_shape(3) ""
    gets $fid npoints
    for {set i 0} {$i<$npoints} {incr i} {
	gets $fid temp
	set temp [tri2coord [lindex $temp 0] [lindex $temp 1]]
	eval "lappend c_shape(3) $temp"
    }
    gets $fid temp
    for {set i 0} {$i<7} {incr i} {
	gets $fid temp
	set special(3,$i) [tri2coord [lindex $temp 0] [lindex $temp 1]]
    }

    close $fid    
}

# Read textual information
proc tritile::init_language {lang} {
    variable text
    variable legend
    variable language

    set language $lang

    set fid [open "language/$lang.txt"]
    gets $fid temp
    for {set i 0} {$i<15} {incr i} {
	gets $fid temp
	gets $fid text($i)
    }
    set text(1) [lindex $text(1) 0]\n[lindex $text(1) 1]
    set text(2) [lindex $text(2) 0]\n[lindex $text(2) 1]

    # ----- Read special points from pi/5 -----
    gets $fid temp
    for {set i 0} {$i<7} {incr i} {
	gets $fid temp
	gets $fid legend(5,$i)
    }

    # ----- Read special points from pi/4 -----
    gets $fid temp
    for {set i 0} {$i<7} {incr i} {
	gets $fid temp
	gets $fid legend(4,$i)
    }

    # ----- Read special points from pi/3 -----
    gets $fid temp
    for {set i 0} {$i<7} {incr i} {
	gets $fid temp
	gets $fid legend(3,$i)
    }

    close $fid
}

proc tritile::init_change_language  {} {
    toplevel .change -bg black
    wm geometry .change =724x768+300-0
    wm overrideredirect .change 1

    set lg "english portuguese french spanish german italian"
    set lgn "English Portugus Franais Espaol Deutsch Italiano"
    
    label .change.title -text "Change Language" -bg black -fg #ffff91  \
	    -font "Helvetica 22 bold"
    place .change.title -x 250 -y 50

    frame .change.all -bg black -bd 5 -relief ridge

    for {set i 0} {$i<6} {incr i} {
	set l [lindex $lg $i]
	frame .change.all.$l -bg black
	image create photo f$l -file images/flags/$l.gif
	button .change.all.$l.but -image f$l -command "set lang $l"
	label .change.all.$l.msg -text [lindex $lgn $i] -bg Black -fg White \
		-font "Helvetica 14 bold"
	pack .change.all.$l.but .change.all.$l.msg -side left -padx 15
	pack .change.all.$l -side top -ipady 15 -ipadx 30 -anchor w
    }

    place .change.all -x 220 -y 180
    
    wm withdraw .change
}

# Init. the windows
proc tritile::init_windows {} {    
    init_top
    init_bottom    
    init_main
    init_about
    init_change_language
}


# -------------------------------------------------------
# Coordinates transformation
# -------------------------------------------------------

# Canvas coords. to Triangle Tiling coords.
proc tritile::coord2tri {x y} {
    variable xdim
    variable ydim
    variable size
    
    set xmid [expr $xdim / 2]
    set ymid [expr $ydim / 2]
    
    set nx [expr ($x-$xmid)/$size.0]
    set ny [expr ($ymid-$y)/$size.0]

    return "$nx $ny"
}

# Triangle Tiling coords. to Canvas coords.
proc tritile::tri2coord {x y} {
    variable xdim
    variable ydim
    variable size
    
    set xmid [expr $xdim / 2]
    set ymid [expr $ydim / 2]

    set nx [expr ($x)*$size+$xmid]
    set ny [expr $ymid-($y)*$size]

    return "$nx $ny"
}


# -------------------------------------------------------
# Shape procedures
# -------------------------------------------------------

# Draw base triangle shape of polihedron
proc tritile::draw_poly {points} {
    set i 0
    foreach {cx cy} $points {
	set c [tri2coord $cx $cy]
	set x$i [lindex $c 0]
	set y$i [lindex $c 1]
	incr i
    }
    .main.shape.c coords orange $x0 $y0 $x6 $y6 $x3 $y3 $x5 $y5
    .main.shape.c coords green $x3 $y3 $x6 $y6 $x1 $y1 $x4 $y4
    .main.shape.c coords magenta $x3 $y3 $x4 $y4 $x2 $y2 $x5 $y5
    
    .main.shape.c coords point [expr $x7-4] [expr $y7-4] \
	    [expr $x7+4] [expr $y7+4]
	
}

# Draw base triangle shape of Sphere
proc tritile::draw_curve {points} {
    global angle

    set i 0
    foreach {cx cy} $points {
    	set c [tri2coord $cx $cy]
    	set x$i [lindex $c 0]
    	set y$i [lindex $c 1]
    	incr i
    }

    if {$angle==4} then {
	.main.shape.c coords orange $x0 $y0 $x1 $y1 $x2 $y2 $x3 $y3 \
		$x7 $y7 $x11 $y11 $x15 $y15 $x14 $y14 $x13 $y13 $x12 $y12 \
		$x8 $y8 $x4 $y4
	.main.shape.c coords green $x16 $y16 $x17 $y17 $x18 $y18 $x19 $y19 \
		$x23 $y23 $x27 $y27 $x31 $y31 $x30 $y30 $x29 $y29 $x28 $y28 \
		$x24 $y24 $x20 $y20
	.main.shape.c coords magenta $x32 $y32 $x33 $y33 $x34 $y34 $x35 $y35 \
		$x39 $y39 $x43 $y43 $x47 $y47 $x46 $y46 $x45 $y45 $x44 $y44 \
		$x40 $y40 $x36 $y36
    } elseif {$angle==5} then {
	.main.shape.c coords orange $x0 $y0 $x1 $y1 $x2 $y2 $x5 $y5 \
		$x8 $y8 $x7 $y7 $x6 $y6 $x3 $y3
	.main.shape.c coords green $x9 $y9 $x10 $y10 $x11 $y11 $x14 $y14 \
		$x17 $y17 $x16 $y16 $x15 $y15 $x12 $y12
	.main.shape.c coords magenta $x18 $y18 $x19 $y19 $x20 $y20 $x23 $y23 \
		$x26 $y26 $x25 $y25 $x24 $y24 $x21 $y21
    } elseif {$angle==3} then {
	.main.shape.c coords orange $x0 $y0 $x1 $y1 $x2 $y2 $x3 $y3 \
		$x4 $y4 $x5 $y5 $x11 $y11 $x17 $y17 $x23 $y23 $x29 $y29 \
		$x35 $y35 $x34 $y34 $x33 $y33 $x32 $y32 $x31 $y31 $x30 $y30 \
		$x24 $y24 $x18 $y18 $x12 $y12 $x6 $y6
	.main.shape.c coords green $x36 $y36 $x37 $y37 $x38 $y38 $x39 $y39 \
		$x40 $y40 $x41 $y41 $x47 $y47 $x53 $y53 $x59 $y59 $x65 $y65 \
		$x71 $y71 $x70 $y70 $x69 $y69 $x68 $y68 $x67 $y67 $x66 $y66 \
		$x60 $y60 $x54 $y54 $x48 $y48 $x42 $y42
	.main.shape.c coords magenta $x72 $y72 $x73 $y73 $x74 $y74 $x75 $y75 \
		$x76 $y76 $x77 $y77 $x83 $y83 $x89 $y89 $x95 $y95 $x101 $y101 \
		$x107 $y107 $x106 $y106 $x105 $y105 $x104 $y104 $x103 $y103 \
		$x102 $y102 $x96 $y96 $x90 $y90 $x84 $y84 $x78 $y78
    }

    set x [lindex $c 0]
    set y [lindex $c 1]
    .main.shape.c coords point [expr $x-4] [expr $y-4] \
	    [expr $x+4] [expr $y+4]
}

# Change shape on predefined refresh rate
proc tritile::shape {} {
    variable delay
    variable px
    variable py
    variable lx
    variable ly
    variable legend
    global type
    global angle

    if {$px != $lx || $py != $ly} then { 
	
	set lx $px
	set ly $py

	set c [coord2tri $px $py]
	
	set points [ChangeShape [lindex $c 0] [lindex $c 1]]

	set legendnum [lindex $points 0]
	set points [lreplace $points 0 0]

	if {$type==1 || $legendnum==-1} {
	    .bottom.legend configure -text ""
	} else {
	    .bottom.legend configure -text $legend($angle,$legendnum)
	}
	
	if {$type==0 || $angle==6} {
	    draw_poly $points
	} else {
	    draw_curve $points
	}
    }

    after $delay "tritile::shape"
}


# -------------------------------------------------------
# Button procedures
# -------------------------------------------------------

# Change simmetry group
proc tritile::group {g} {
    variable px
    variable py
    variable lx
    variable c_shape
    variable special
    variable text

    set deg [expr 180/$g]
    .main.shape.c itemconfigure angle1 -text "/ $g $text(14) = ${deg}"
    .main.shape.c itemconfigure sum \
	    -text "${deg}\n60\n+ 90\n[expr $deg+60+90]"

    if {$g==6} then {
	pack forget .main.angle.poli.b .main.angle.poli.l \
		.main.angle.sphere.b .main.angle.sphere.l
	.main.shape.c coords shape -1 -1 -2 -2 -2 -1
	for {set i 0} {$i < 7} {incr i} {
	    .main.shape.c coords sp$i -1 -1 -2 -2
	}

	.main.shape.c coords pi_blue 30 30
	.main.shape.c coords angle1 43 23
	.main.shape.c coords angle3 178 255
    } else {
	pack .main.angle.poli.b .main.angle.poli.l -side top
	pack .main.angle.sphere.b .main.angle.sphere.l -side top
	eval ".main.shape.c coords shape $c_shape($g)"
	for {set i 0} {$i < 7} {incr i} {
	    set x [lindex $special($g,$i) 0]
	    set y [lindex $special($g,$i) 1]
	    .main.shape.c coords sp$i [expr $x-3] [expr $y-3] \
		    [expr $x+3] [expr $y+3]
	}
	if {$g==3} {
	    .main.shape.c coords pi_blue 30 40
	    .main.shape.c coords angle1 43 33
	    .main.shape.c coords angle3 257 255
	} elseif {$g==4} {
	    .main.shape.c coords pi_blue 30 47
	    .main.shape.c coords angle1 43 40
	    .main.shape.c coords angle3 215 255
	} else {
	    .main.shape.c coords pi_blue 30 40
	    .main.shape.c coords angle1 43 33
	    .main.shape.c coords angle3 193 255
	}
	
    }

    set temp [ChangeGroup $g]
    set temp [eval "tri2coord $temp"] 
    set px [expr [lindex $temp 0]/1]
    set py [expr [lindex $temp 1]/1]
    .main.angle.poli.b invoke
    set lx -1
}

# Change to polihedron or sphere
proc tritile::curve {num} {
    variable lx
    variable px
    variable py

    ChangeCurve $num
    set lx -1
}

# Reset
proc tritile::reset {} {
    variable px
    variable py
    
    .main.angle.pi4 invoke
    .main.see.both.b invoke
    
    puts "(ui-target world)"
    
    set px 110
    set py 170
}

# Show about window
proc tritile::about {} {
    wm withdraw .top
    wm withdraw .bottom
    wm deiconify .about
    grab .about
}

# Change Language
proc tritile::change_language {} {
    variable language
    variable text
    variable lx
    global angle
    global lang

    wm withdraw .top
    wm withdraw .bottom
    wm deiconify .change
    grab .change

    tkwait variable lang

    if {[string compare $lang german]==0 && \
	    [string compare $language german]!=0} {
	.main.shape.t configure -font "Helvetica 12 bold" -width 41
	.main.see.t configure -font "Helvetica 12 bold" -width 41
	.bottom.legend configure -font "Helvetica 14 bold"
    } elseif {[string compare $language german]==0 && \
	    [string compare $lang german]!=0} {
	.main.shape.t configure -font "Helvetica 16 bold" -width 32
	.main.see.t configure -font "Helvetica 16 bold" -width 32
	.bottom.legend configure -font "Helvetica 16 bold"
    }

    set language $lang
    init_language $language


    image create photo flag -file images/flags/$language.gif
    .bottom.lg.flag configure -image flag

    set deg [expr 180/$angle]
    .main.shape.c itemconfigure angle1 -text "/ $angle $text(14) = ${deg}"
    set lx -1

    grab release .change;
    wm withdraw .change;
    wm deiconify .top;
    wm deiconify .bottom
}

# -------------------------------------------------------
# Main program
# -------------------------------------------------------
bind all <Escape> "puts (quit); exit"

InitAll
tritile::init_language "english"
tritile::init_shape
tritile::init_windows

wm withdraw .
